//-------------------------------------------------------------------------------------------------
// Rgba.h (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_Rgba_
#define _include_Rgba_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.h"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Rgba
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class Rgba
{
public:
	static const Rgba WHITE;
	static const Rgba BLACK;
	static const Rgba RED;
	static const Rgba GREEN;
	static const Rgba BLUE;
	static const Rgba YELLOW;
	static const Rgba PURPLE;
	static const Rgba CYAN;
	static const Rgba ORANGE;
	static const Rgba GREY;
	static const Rgba DARKRED;
	static const Rgba DARKGREEN;
	static const Rgba DARKBLUE;
	static const Rgba DARKYELLOW;
	static const Rgba DARKPURPLE;
	static const Rgba DARKCYAN;
	static const Rgba DARKORANGE;
	static const Rgba DARKGREY;

	float r; // these are an exception to the "no-public-members" rule of thumb
	float g; // ...for the sake of clarity and brevity in use throughout the code
	float b;
	float a;

	/// Construction / Destruction
	~Rgba(); // destructor
	Rgba(); // default constructor
	Rgba( const Rgba& rgba ); // copy constructor
	Rgba( const float red, const float green, const float blue, const float alpha = 1.0f );

	/// Interface methods
	const float CalcLengthSquared() const;
	const float CalcLength() const;
	const float CalcLengthRGB() const;
	const float Normalize();
	void SetRGBA( const float red, const float green, const float blue, const float alpha = 1.0f );
	void GetRGBA( float& red, float& green, float& blue, float& alpha ) const;
	void GetRGB( float& red, float& green, float& blue ) const;
	void SetRGBABytes( byte_t red, byte_t green, byte_t blue, byte_t alpha = (byte_t)0xff );
	void GetRGBABytes( byte_t& red, byte_t& green, byte_t& blue, byte_t& alpha ) const;
	void GetRGBBytes( byte_t& red, byte_t& green, byte_t& blue ) const;
	const float* GetAsFloatArray() const;
	const Rgba& Clamp( const Rgba& min, const Rgba& max );
	const float CalcMaxComponent() const;
	const Rgba Lerp( const Rgba& rhs, const float rhs_fraction );
	float ByteToFloat( byte_t b ) const;
	byte_t FloatToByte( float f ) const;

	/// Operators
	const Rgba& operator = ( const Rgba& rhs );
	const Rgba& operator += ( const Rgba& rhs );
	const Rgba& operator -= ( const Rgba& rhs );
	const Rgba& operator *= ( const float scale );
	const Rgba& operator /= ( const float divisor );

	const Rgba operator - () const;
	const Rgba operator - ( const Rgba& rhs ) const;
	const Rgba operator + ( const Rgba& rhs ) const;
	const Rgba operator * ( const float scale ) const;
	const float operator | ( const Rgba& rhs ) const; // dot product

	const float operator [] ( const int index ) const;
	float& operator [] ( const int index );

	const bool operator == ( const Rgba& rhs ) const;
	const bool operator != ( const Rgba& rhs ) const;
};

//-------------------------------------------------------------------------------------------------
// External Operators & Functions
//-------------------------------------------------------------------------------------------------
const Rgba operator * ( const float scale, const Rgba& rhs );
const bool EpsilonEqual( const Rgba& lhs, const Rgba& rhs, const float epsilon );


//-------------------------------------------------------------------------------------------------
// Rgba::~Rgba()
//-------------------------------------------------------------------------------------------------
inline Rgba::~Rgba()
{
}


//-------------------------------------------------------------------------------------------------
// Rgba::Rgba() - default constructor
//-------------------------------------------------------------------------------------------------
inline Rgba::Rgba()
{
	// do nothing - keep this class as flyweight as possible
}


//-------------------------------------------------------------------------------------------------
// Rgba::Rgba( const Rgba& rgba ) - copy constructor
//-------------------------------------------------------------------------------------------------
inline Rgba::Rgba( const Rgba& rgba )
:
r( rgba.r ),
g( rgba.g ),
b( rgba.b ),
a( rgba.a )
{
}


//-------------------------------------------------------------------------------------------------
// Rgba( float, float, float, float )
//-------------------------------------------------------------------------------------------------
inline Rgba::Rgba( const float red, const float green, const float blue, const float alpha )
:
r( red ),
g( green ),
b( blue ),
a( alpha )
{
}


//-------------------------------------------------------------------------------------------------
// operator =
//-------------------------------------------------------------------------------------------------
inline const Rgba& Rgba::operator = ( const Rgba& rhs )
{
	if( this == &rhs )
		return *this;

	r = rhs.r;
	g = rhs.g;
	b = rhs.b;
	a = rhs.a;
	return *this;
}


//-------------------------------------------------------------------------------------------------
// operator +=
//-------------------------------------------------------------------------------------------------
inline const Rgba& Rgba::operator += ( const Rgba& rhs )
{
	r += rhs.r;
	g += rhs.g;
	b += rhs.b;
	a += rhs.a;
	return *this;
}


//-------------------------------------------------------------------------------------------------
// operator -=
//-------------------------------------------------------------------------------------------------
inline const Rgba& Rgba::operator -= ( const Rgba& rhs )
{
	r -= rhs.r;
	g -= rhs.g;
	b -= rhs.b;
	a -= rhs.a;
	return *this;
}


//-------------------------------------------------------------------------------------------------
// operator *=
//-------------------------------------------------------------------------------------------------
inline const Rgba& Rgba::operator *= ( const float scale )
{
	r *= scale;
	g *= scale;
	b *= scale;
	a *= scale;
	return *this;
}


//-------------------------------------------------------------------------------------------------
// operator /=
//-------------------------------------------------------------------------------------------------
inline const Rgba& Rgba::operator /= ( const float divisor )
{
	if( divisor )
	{
		r /= divisor;
		g /= divisor;
		b /= divisor;
		a /= divisor;
	}
	else
	{
		r = 0.0f;
		g = 0.0f;
		b = 0.0f;
		a = 0.0f;
	}
	return *this;
}


//-------------------------------------------------------------------------------------------------
// operator - (unary minus)
//-------------------------------------------------------------------------------------------------
inline const Rgba Rgba::operator - () const
{
	return Rgba( -r, -g, -b, -a );
}


//-------------------------------------------------------------------------------------------------
// operator -
//-------------------------------------------------------------------------------------------------
inline const Rgba Rgba::operator - ( const Rgba& rhs ) const
{
	return( Rgba( r - rhs.r, g - rhs.g, b - rhs.b, g - rhs.a ) );
}


//-------------------------------------------------------------------------------------------------
// operator +
//-------------------------------------------------------------------------------------------------
inline const Rgba Rgba::operator + ( const Rgba& rhs ) const
{
	return( Rgba( r + rhs.r, g + rhs.g, b + rhs.b, a + rhs.a ) );
}


//-------------------------------------------------------------------------------------------------
// operator *
//-------------------------------------------------------------------------------------------------
inline const Rgba Rgba::operator * ( const float scale ) const
{
	return( Rgba( r * scale, g * scale, b * scale, a * scale ) );
}


//-------------------------------------------------------------------------------------------------
// operator [] const
//-------------------------------------------------------------------------------------------------
inline const float Rgba::operator [] ( const int index ) const
{
//	ASSERT( ( index >= 0 ) && ( index < 4 ) );
	return ( &r )[ index ];
}


//-------------------------------------------------------------------------------------------------
// operator []
//-------------------------------------------------------------------------------------------------
inline float& Rgba::operator [] ( const int index )
{
//	ASSERT( ( index >= 0 ) && ( index < 4 ) );
	return ( &r )[ index ];
}


//-------------------------------------------------------------------------------------------------
// operator ==
//-------------------------------------------------------------------------------------------------
inline const bool Rgba::operator == ( const Rgba& rhs ) const
{
	return( (r == rhs.r) && (g == rhs.g) && (b == rhs.b) && (a == rhs.a) );
}


//-------------------------------------------------------------------------------------------------
// operator !=
//-------------------------------------------------------------------------------------------------
inline const bool Rgba::operator != ( const Rgba& rhs ) const
{
	return( (r != rhs.r) || (g != rhs.g) || (b != rhs.b) || (a != rhs.a) );
}


//-------------------------------------------------------------------------------------------------
// CalcLengthSquared
//-------------------------------------------------------------------------------------------------
inline const float Rgba::CalcLengthSquared() const
{
	return( r*r + g*g + b*b + a*a );
}


//-------------------------------------------------------------------------------------------------
// CalcLength
//-------------------------------------------------------------------------------------------------
inline const float Rgba::CalcLength() const
{
	return( (float) sqrt( r*r + g*g + b*b + a*a ) );
}


//-------------------------------------------------------------------------------------------------
// Normalize
//-------------------------------------------------------------------------------------------------
inline const float Rgba::Normalize()
{
	float length = CalcLength();
	if( length )
	{
		r /= length;
		g /= length;
		b /= length;
		a /= length;
	}

	return( length );
}


//-------------------------------------------------------------------------------------------------
// SetRGBA
//-------------------------------------------------------------------------------------------------
inline void Rgba::SetRGBA( const float red, const float green, const float blue, const float alpha )
{
	r = red;
	g = green;
	b = blue;
	a = alpha;
}


//-------------------------------------------------------------------------------------------------
// SetRGBABytes
//-------------------------------------------------------------------------------------------------
inline void Rgba::SetRGBABytes( byte_t red, byte_t green, byte_t blue, byte_t alpha )
{
	r = ByteToFloat( red );
	g = ByteToFloat( green );
	b = ByteToFloat( blue );
	a = ByteToFloat( alpha );
}


//-------------------------------------------------------------------------------------------------
// GetRGBABytes
//-------------------------------------------------------------------------------------------------
inline void Rgba::GetRGBABytes( byte_t& red, byte_t& green, byte_t& blue, byte_t& alpha ) const
{
	red		= FloatToByte( r );
	green	= FloatToByte( g );
	blue	= FloatToByte( b );
	alpha	= FloatToByte( a );
}


//-------------------------------------------------------------------------------------------------
// GetRGBBytes
//-------------------------------------------------------------------------------------------------
inline void Rgba::GetRGBBytes( byte_t& red, byte_t& green, byte_t& blue ) const
{
	red		= FloatToByte( r );
	green	= FloatToByte( g );
	blue	= FloatToByte( b );
}


//-------------------------------------------------------------------------------------------------
// GetRGBA
//-------------------------------------------------------------------------------------------------
inline void Rgba::GetRGBA( float& red, float& green, float& blue, float& alpha ) const
{
	red		= r;
	green	= g;
	blue	= b;
	alpha	= a;
}


//-------------------------------------------------------------------------------------------------
// GetRGB
//-------------------------------------------------------------------------------------------------
inline void Rgba::GetRGB( float& red, float& green, float& blue ) const
{
	red		= r;
	green	= g;
	blue	= b;
}


//-------------------------------------------------------------------------------------------------
// GetAsFloatArray
//-------------------------------------------------------------------------------------------------
inline const float* Rgba::GetAsFloatArray() const
{
	return (float*) this;
}


//-------------------------------------------------------------------------------------------------
// Clamp
//-------------------------------------------------------------------------------------------------
inline const Rgba& Rgba::Clamp( const Rgba& min, const Rgba& max )
{
	/// Clamp R component
	if( r > max.r )
		r = max.r;
	else if( r < min.r )
		r = min.r;

	/// Clamp G component
	if( g > max.g )
		g = max.g;
	else if( g < min.g )
		g = min.g;

	/// Clamp B component
	if( b > max.b )
		b = max.b;
	else if( b < min.b )
		b = min.b;

	/// Clamp A component
	if( a > max.a )
		a = max.a;
	else if( a < min.a )
		a = min.a;

	return *this;
}


//-------------------------------------------------------------------------------------------------
// CalcMaxComponent
//-------------------------------------------------------------------------------------------------
inline const float Rgba::CalcMaxComponent() const
{
	/// Assume r is the largest
	float max = r;

	/// Check if g is bigger
	if( g > max )
		max = g;

	/// Check if b is bigger
	if( b > max )
		max = b;

	/// Check if a is bigger
	if( a > max )
		max = a;

	return( max );
}


//-------------------------------------------------------------------------------------------------
// Lerp
//-------------------------------------------------------------------------------------------------
inline const Rgba Rgba::Lerp( const Rgba& rhs, const float rhs_fraction )
{
	return Rgba( r + rhs_fraction * (rhs.r - r),
					g + rhs_fraction * (rhs.g - g),
					b + rhs_fraction * (rhs.b - b),
					a + rhs_fraction * (rhs.a - a));
}


//-------------------------------------------------------------------------------------------------
// ByteToFloat
//-------------------------------------------------------------------------------------------------
inline float Rgba::ByteToFloat( byte_t b ) const
{
	return( float( b ) * (1.0f / 255.0f) );
}


//-------------------------------------------------------------------------------------------------
// FloatToByte
//-------------------------------------------------------------------------------------------------
inline byte_t Rgba::FloatToByte( float f ) const
{
	return( (byte_t) (f * 255.0f) );
}


//-------------------------------------------------------------------------------------------------
// operator * (scale, Rgba)
//-------------------------------------------------------------------------------------------------
inline const Rgba operator * ( const float scale, const Rgba& rhs )
{
	return( Rgba( scale * rhs.r, scale * rhs.g, scale * rhs.b, scale * rhs.a ) );
}


//-------------------------------------------------------------------------------------------------
// EpsilonEqual
//-------------------------------------------------------------------------------------------------
inline const bool EpsilonEqual( const Rgba& lhs, const Rgba& rhs, const float epsilon )
{
	return( fabs( lhs.r - rhs.r ) < epsilon &&
			fabs( lhs.g - rhs.g ) < epsilon &&
			fabs( lhs.b - rhs.b ) < epsilon &&
			fabs( lhs.a - rhs.a ) < epsilon );
}


#endif // _include_Rgba_
