//-------------------------------------------------------------------------------------------------
// TheMessageHandler.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.h"
#include "TheMessageHandler.h"
#include "TheInputSystem.h"
#include "TheVideoSystem.h"
#include "TheConsole.h"
#include "TheApplication.h"
#include "engineAssert.h"


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC TheMessageHandler*	TheMessageHandler::_theMessageHandler = NULL;
STATIC bool					TheMessageHandler::_isConstructed = false;
STATIC bool					TheMessageHandler::_isInitialized = false;



//-------------------------------------------------------------------------------------------------
// constructor
//-------------------------------------------------------------------------------------------------
TheMessageHandler::TheMessageHandler()
{
	/// Make sure the singleton object doesn't get constructed twice
	VALIDATE( !_isConstructed );

	/// Set construction complete
	_isConstructed = true;
}


//-------------------------------------------------------------------------------------------------
// destructor
//-------------------------------------------------------------------------------------------------
TheMessageHandler::~TheMessageHandler()
{
	VALIDATE( !_isInitialized && _isConstructed );

	/// Set un-constructed
	_isConstructed = false;
}


//-------------------------------------------------------------------------------------------------
// CreateSingleton
//-------------------------------------------------------------------------------------------------
STATIC TheMessageHandler* TheMessageHandler::CreateSingleton()
{
	VALIDATE( !_isConstructed );
	VALIDATE( !_isInitialized );
	VALIDATE( !_theMessageHandler );

	/// Create the singleton and set the static pointer
	_theMessageHandler = new TheMessageHandler;

	return _theMessageHandler;
}


//-------------------------------------------------------------------------------------------------
// Singleton
//-------------------------------------------------------------------------------------------------
STATIC TheMessageHandler* TheMessageHandler::Singleton()
{
	ASSERT( _isConstructed );
	ASSERT( _theMessageHandler );

	return _theMessageHandler;
}


//-------------------------------------------------------------------------------------------------
// DestroySingleton
//-------------------------------------------------------------------------------------------------
STATIC void TheMessageHandler::DestroySingleton()
{
	VALIDATE( _isConstructed );
	VALIDATE( !_isInitialized );
	VALIDATE( _theMessageHandler );

	/// Destroy the singleton and null the static pointer
	delete _theMessageHandler;
	_theMessageHandler = NULL;
}


//-------------------------------------------------------------------------------------------------
// Init
//-------------------------------------------------------------------------------------------------
void TheMessageHandler::Init()
{
	CONSOLE->Printf( "::: Initialization : Message Handler (starting)\n" );

	/// Make sure the singleton object doesn't get initialized twice
	VALIDATE( this && _isConstructed && !_isInitialized );

	/// Set initialization complete
	_isInitialized = true;
	CONSOLE->Printf( "::: Initialization : Message Handler (finished)\n" );
}


//-------------------------------------------------------------------------------------------------
// Shutdown
//-------------------------------------------------------------------------------------------------
void TheMessageHandler::Shutdown()
{
	CONSOLE->Printf( "::: Shutdown : Message Handler (starting)\n" );

	/// De-initialize the system
	_isInitialized = false;
	CONSOLE->Printf( "::: Shutdown : Message Handler (finished)\n" );
}


//-------------------------------------------------------------------------------------------------
// HandleWin32Message
// 
// WinProc() - the windows message handling procedure - calls this function before doing
//	anything else to allow the application an opportunity to handle each Win32 WM_XXXX-style
//	message as it comes in.  Return value is true if the message was handled (or trapped)
//	internally by the application; returns false if not (in which case the message
//	will be passed on to Windows for further processing via DefWindowProc()).
//-------------------------------------------------------------------------------------------------
const bool TheMessageHandler::HandleWin32Message(
	const HWND hWnd, const UINT messageID, const WPARAM wParam, const LPARAM lParam )
{
	ASSERT( this );
	ASSERT( _isConstructed );
	ASSERT( _isInitialized );

	/// FIXME: not yet implemented!
	UNUSED_ARG hWnd;
	UNUSED_ARG wParam;
	UNUSED_ARG lParam;

	bool wasHandled = false;

	switch( messageID )
	{
		case WM_CLOSE:
			PostQuitMessage( 0 );
			break;
		case WM_KEYDOWN:
		case WM_KEYUP:
		case WM_SYSKEYDOWN:
		case WM_SYSKEYUP:
		case 0x020A: // WM_MOUSEWHEEL
			wasHandled = INPUT->ProcessInputEvent( messageID, wParam, lParam );
			break;
		case WM_LBUTTONDOWN:
			wasHandled = INPUT->ProcessInputEvent( WM_KEYDOWN, VK_LBUTTON, 0 );
			break;
		case WM_LBUTTONUP:
			wasHandled = INPUT->ProcessInputEvent( WM_KEYUP, VK_LBUTTON, 0 );
			break;
		case WM_RBUTTONDOWN:
			wasHandled = INPUT->ProcessInputEvent( WM_KEYDOWN, VK_RBUTTON, 0 );
			break;
		case WM_RBUTTONUP:
			wasHandled = INPUT->ProcessInputEvent( WM_KEYUP, VK_RBUTTON, 0 );
			break;
		case WM_MBUTTONDOWN:
			wasHandled = INPUT->ProcessInputEvent( WM_KEYDOWN, VK_MBUTTON, 0 );
			break;
		case WM_MBUTTONUP:
			wasHandled = INPUT->ProcessInputEvent( WM_KEYUP, VK_MBUTTON, 0 );
			break;
		case WM_MOVE:
		{
			int newClientX = (int)(short) LOWORD( lParam );
			int newClientY = (int)(short) HIWORD( lParam );
			VIDEO->NotifyOfWindowMove( newClientX, newClientY );
			break;
		}
		case WM_ACTIVATEAPP:
			if( wParam == TRUE )
			{
				/// App just gained focus
				APPLICATION->OnAppGainFocus();
			}
			else
			{
				/// App just lost focus
				APPLICATION->OnAppLoseFocus();
			}
			break;
		default:
			break;
	}

	return wasHandled;
}



