//-------------------------------------------------------------------------------------------------
// Timer.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.h"
#include "Timer.h"
#include "engineAssert.h"


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC double Timer::s_queryPerfSecsPerCount = 0.0;
STATIC LARGE_INTEGER Timer::s_queryPerfOffset;


//-------------------------------------------------------------------------------------------------
// InitPerformanceCounter
//-------------------------------------------------------------------------------------------------
STATIC void Timer::_InitPerformanceCounter()
{
	/// Get the system's perf-ticks-per-second ratio
	LARGE_INTEGER queryPerfCountsPerSec;
	BOOL success = QueryPerformanceFrequency( &queryPerfCountsPerSec );

	/// Make sure the performance counter responded correctly
	VALIDATE( success );
	VALIDATE( queryPerfCountsPerSec.QuadPart );

	/// Calculate the seconds-per-tick ratio (inverse)
	s_queryPerfSecsPerCount = (double) 1.0 / (double) queryPerfCountsPerSec.QuadPart;

	/// Get the current perf count time, use this as a starting baseline against all
	// subsequent times.  This helps prevent time precision degradation on computers
	// who have been "up" a long time (perf counter counts up from last reboot).
	// Note that since GetSysTimeSeconds() returns a double precision float this
	// is usually a non-issue (as a computer would have to be up for over a thousand
	// years before losing significant precision).  However, clients who cast this
	// result to a float will suffer noticeable degradation on a computer that has
	// been up for as little as 1 or 2 days.  To combat this, we sample the perf
	// counter on Initialization and offset subsequent readings from this baseline
	// so that "1 or 2 days of OS up-time" becomes "1 or 2 days since the application
	// was started" (which is much more reasonable).
	QueryPerformanceCounter( &s_queryPerfOffset );
}


//-------------------------------------------------------------------------------------------------
// GetSysTimeSeconds
// 
// NOTE: it is HIGHLY recommended that you use double, and not float, for storing time in
//	seconds.  Single-precision float runs into noticeable degradation very quickly,
//	becoming extremely noticeable 
//-------------------------------------------------------------------------------------------------
STATIC double Timer::GetSysTimeSeconds()
{
	/// Check if the perf timer needs initializing
	if( s_queryPerfSecsPerCount == 0.0 )
	{
		_InitPerformanceCounter();
	}

	/// Calculate precise seconds using perf timer
	LARGE_INTEGER queryPerfCount;
	QueryPerformanceCounter( &queryPerfCount );
	queryPerfCount.QuadPart -= s_queryPerfOffset.QuadPart;
	double seconds = s_queryPerfSecsPerCount * (double) queryPerfCount.QuadPart;
	return seconds;
}


//-------------------------------------------------------------------------------------------------
// GetSysTimeMS
//-------------------------------------------------------------------------------------------------
STATIC dword_t Timer::GetSysTimeMS()
{
	double ms = 1000.0 * GetSysTimeSeconds();
	return (dword_t) ms;
}


